package org.example;

import jakarta.xml.bind.JAXBElement;
import org.springframework.http.MediaType;
import org.springframework.http.codec.xml.Jaxb2XmlDecoder;
import org.springframework.http.codec.xml.Jaxb2XmlEncoder;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.ExchangeStrategies;
import org.springframework.web.reactive.function.client.WebClient;

import javax.xml.namespace.QName;

@Service
public class Api {
    private final WebClient webClient;

    public Api(WebClient.Builder webclientBuilder) {
        final ExchangeStrategies strategies = ExchangeStrategies
                .builder()
                .codecs(codecs -> {
                    codecs.defaultCodecs().jaxb2Encoder(new Jaxb2XmlEncoder());
                    codecs.defaultCodecs().jaxb2Decoder(new Jaxb2XmlDecoder());
                }).build();

        this.webClient = webclientBuilder.clone()
                .baseUrl("http://localhost:8082")
                .exchangeStrategies(strategies)
                .build();
    }

    public void call() {
        Pojo pojo = new Pojo("test");
        JAXBElement<Pojo> pojoJaxB = new JAXBElement<>(new QName("any"), Pojo.class, pojo);

        this.webClient.post()
                .contentType(MediaType.APPLICATION_XML)
                .bodyValue(pojoJaxB)
                .retrieve()
                .toBodilessEntity()
                .block();
    }
}
